unit osNumericEdit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, osTypes, osEdit,registry;

type
  TosNumericEdit = class(TosEdit)
  private
    FAboutBox:String;
    FDecimalDigits:Integer;
    FAllowDecimal: Boolean;
    FAllowNegative: Boolean;
    FHexaDecimal: Boolean;
    FBeepOnError: Boolean;
    Decimal: Boolean;
    FValue:Extended;
    FForceDecimal:Boolean;
  protected
    procedure Change; override;
    procedure KeyPress(var Key: Char); override;
    procedure SetValue(Value: Extended);
    procedure SetIntValue(Value: Integer);
    procedure SetInt64Value(Value:Int64);
    function GetIntValue: Integer;
    function GetInt64Value:Int64;
    procedure SetHexadecimal(Value:Boolean);
    procedure SetAllowDecimal(Value:Boolean);
    procedure SetAllowNegative(Value:Boolean);
    procedure SetDecimalDigits(Value:Integer);
    procedure SetForceDecimal(Value:Boolean);
//    function DecimalSeparator:String;
    procedure DoExit;override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    property Int64Value: Int64 read GetInt64Value write SetInt64Value default 0;
  published
    property About : String read FAboutBox write FAboutBox;
    property BeepOnError: Boolean read FBeepOnError write FBeepOnError default False;
    property Value: Extended read FValue write SetValue;
    property IntValue: Integer read GetIntValue write SetIntValue default 0;
    property AutoSelect;
    property AllowDecimal: Boolean read FAllowDecimal write SetAllowDecimal default False;
    property AllowNegative: Boolean read FAllowNegative write SetAllowNegative default False;
    property Hexadecimal: Boolean read FHexadecimal write SetHexadecimal default False;
    property DecimalDigits:Integer read FDecimalDigits write SetDecimalDigits;
    property ForceDecimal:Boolean read FForceDecimal write SetForceDecimal default false;
    property Alignment default taRightJustify;
    property Hint;
    property ShowHint;
    property Text stored false;
  end;

implementation

//---------------------------------------------------------------------------

constructor TosNumericEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Width := 41;
  AllowDecimal := False;
  AllowNegative := False;
  Hexadecimal := False;
  BeepOnError := False;
  Alignment:=taRightJustify;
  FDecimalDigits:=-1;
  Value:=0;
end;

//---------------------------------------------------------------------------

destructor TosNumericEdit.destroy;
begin
  inherited destroy;
end;

//---------------------------------------------------------------------------

procedure TosNumericEdit.DoExit;
begin
  SetValue(Self.Value);
  Inherited DoExit;
end;

//---------------------------------------------------------------------------

procedure TosNumericEdit.SetAllowDecimal (Value:Boolean);
begin
  if FHexadecimal then
    FAllowDecimal:=False
  else
  begin
    FAllowDecimal:=Value;
  end;
end;


procedure TosNumericEdit.SetAllowNegative (Value:Boolean);
begin
  if FHexadecimal then
    FAllowNegative:=False
  else
    FAllowNegative:=Value;
end;


procedure TosNumericEdit.SetHexadecimal(Value:Boolean);
begin
  if Value then
  begin
    FAllowDecimal:=False;
    FAllowNegative:=False;
  end;
  FHexadecimal:=Value;
end;

procedure TosNumericEdit.SetForceDecimal(Value:Boolean);
begin
  if Value=FForceDecimal then exit;
  FForceDecimal:=Value;
  SetValue(self.Value);
end;


procedure TosNumericEdit.KeyPress(var Key: Char);
begin
  key:=upcase(key);
  if not Hexadecimal then
    begin
      if ((key<'-') or (Key<'0') or (key>'9')) and (key<>#8) and (key<>#12)
        and (key<>#13) and (Key<>'-') and (Key<>DecimalSeparator)then
           key:=#0;
      if (key='-') and (SelStart<>0)then
         key:=#0;
      if (key='-') and (AllowNegative=False) then
         key:=#0;
      if (key='-') and (Copy(Text,1,1)='-') and (SelLength=0) then
         key:=#0;
      if (key='-') and (SelStart=0) and (SelLength=0) and (Copy(Text,1,1)<>'-')then
      begin
        text:=key+text;
        if Copy(Text,2,1)<>'0' then
          selstart:=selstart+1;
        key:=#0;
      end;
      if (key=DecimalSeparator) and (decimal=true) and (SelLength=0) then
        key:=#0;
      if (key=DecimalSeparator) and (FAllowDecimal=False) then
        key:=#0;
      if (key='0') and (selstart=0) and (SelLength=0)and (Copy(Text,1,1)<>DecimalSeparator) then
        key:=#0;
      if ((Key<='9') and (Key>='0')) and (selStart=1) and (Copy(Text,1,1)='0') then
        key:=#0;
      if (FDecimalDigits>-1) and (pos(DecimalSeparator,Text)<>0) then
        if(Key<='9') and (Key>='0') then
          if (length(text)-pos(DecimalSeparator,Text)+1>FDecimalDigits) and
            (SelStart>=pos(DecimalSeparator,Text))and (SelLength=0)then
             key:=#0;
    end
  else
    begin
      if ((key<'0')or(key>'9')) and ((key<'A') or (key>'F')) and (key<>chr(8))
        and (key<>chr(12)) and (key<>chr(13)) then key:=chr(0);
   end;

  if FBeepOnError and (key=#0) then beep;
  inherited KeyPress(Key);
end;

//---------------------------------------------------------------------------

procedure TosNumericEdit.Change;
var
texto:String;
begin
  // if csReading in ComponentState then exit;

   texto:=uppercase(text);
   decimal:=false;
   if pos(DecimalSeparator,texto)<>0 then
     decimal:=true;

   if texto='' then
   begin
     text:='0';
     self.SelectAll;
   end;

   if texto='-' then
   begin
     text:='-0';
     SelStart:=1;
     SelLength:=1;
   end;


   if text='-0' then
   begin
     selstart:=1;
     sellength:=1;
   end;

   if Copy(Text,Length(Text),1)<>DecimalSeparator then
     FValue:= StrToFloat(Text)
   else
     FValue:=StrToFloat(Copy(Text,1,Length(Text)-1));

   inherited Change;
end;

//---------------------------------------------------------------------------
procedure TosNumericEdit.SetDecimalDigits(Value:Integer);
begin
  if Value<-1 then Value:=-1;
  FDecimalDigits:=Value;
  SetValue(self.Value);
end;

//---------------------------------------------------------------------------

procedure TosNumericEdit.SetValue(Value: Extended);
var
resultado:String;
f:integer;
begin
  if not Hexadecimal then     
    try
      if not AllowDecimal then
        Text:=inttostr(Round(Value))
      else
      begin
        if FDecimalDigits>-1 then
        begin
          if FForceDecimal then
            Text:= FormatFloat('0.'+StringOfChar('0',FDecimalDigits),Value)
          else
            Text:= FormatFloat('0.'+StringOfChar('#',FDecimalDigits),Value);
        end
        else
          Text:= floattostr(value)
      end;
    except
      if BeepOnError then Beep;
    end
  else
    try
     resultado:=inttohex(Round(Value),16);
     f:=1;
     while (copy(resultado,f,2)='00') and (f<length(resultado)-2) do
        Inc(f,2);
     Text := copy(resultado,f,length(resultado)-f+1);
    except
     if BeepOnError then Beep;
    end;
end;

//---------------------------------------------------------------------------

procedure TosNumericEdit.SetIntValue(Value: Integer);
var f:Integer;
resultado:String;
begin
 if Hexadecimal = False then
    try
     Text:= inttostr(value)
    except
     if BeepOnError then Beep;
    end
  else
    try
     resultado:=inttohex(Value,16);
     f:=1;
     while (copy(resultado,f,2)='00') and (f<length(resultado)-2) do
        Inc(f,2);
     Text := copy(resultado,f,length(resultado)-f+1);
    except
     if BeepOnError then Beep;
    end;
end;

//---------------------------------------------------------------------------

function TosNumericEdit.GetIntValue: Integer;
begin
  result := Round(Self.Value);
end;

//---------------------------------------------------------------------------

procedure TosNumericEdit.SetInt64Value(Value: Int64);
var f:Integer;
resultado:String;
begin
 if Hexadecimal = False then
    try
     Text:= inttostr(value)
    except
     if BeepOnError then Beep;
    end
  else
    try
     resultado:=inttohex(Value,16);
     f:=1;
     while (copy(resultado,f,2)='00') and (f<length(resultado)-2) do
        Inc(f,2);
     Text := copy(resultado,f,length(resultado)-f+1);
    except
     if BeepOnError then Beep;
    end;
end;

//---------------------------------------------------------------------------

function TosNumericEdit.GetInt64Value: Int64;
begin
  result := Round(Self.Value);
end;
end.

